(*
   AnotherReg
   Registry-aware Components Collection
   (C) Glen Why, 1996
   Version 01.00.01

   Version History

   01.00.00 - first
   01.00.01 - Small bug in TVarinatProfile.SetValue fixed

*)
unit AnotherReg;

interface
uses
 Windows, Classes, ExtCtrls, SysUtils;


type

  ERegistryError = class( Exception );

  TRegistryEntry = class( TComponent )
  protected
    function GetExist :Boolean; virtual; abstract;
    procedure DoDelete; virtual; abstract;
  public
    procedure Delete;
    property Exist :Boolean read GetExist;
  end;

  TCustomKey = class( TRegistryEntry )
  private
    FParentKey :TCustomKey;
    FAutoCloseTimer :TTimer;
    FHandle :THandle;
    procedure SetParentKey( V :TCustomKey );
    procedure SetActive( V :Boolean );
    function GetActive :Boolean;
    function GetHandle :THandle;
    procedure SetAutoCloseTimer;
    procedure KillAutoCloseTimer;
    procedure AutoClose( sender :TObject );
    function GetFullPath :string;
  protected
    function GetExist :Boolean; override;
    procedure Notification( AComponent :TComponent;
     Operation :TOperation ); override;
    procedure DoDelete; override;
    function GetPath :string; virtual;
    procedure Open; virtual;
    procedure Close; virtual;
    property ParentKey :TCustomKey read FParentKey write SetParentKey;
  public
    constructor Create( anOwner :TComponent ); override;
    destructor Destroy; override;
    property Active :Boolean read GetActive write SetActive stored false;
    property Handle :THandle read GetHandle;
    property Path :String read GetPath;
    property FullPath :String read GetFullPath;
  end;

  TRegKey = class( TCustomKey )
  published
    property ParentKey;
  end;

  TAppConfigKey = class( TCustomKey )
  private
    FCompanyName :String;
    FApplicationName :String;
    FApplicationVersion :String;
    procedure SetApplicationName( V :String );
    procedure SetCompanyName( V :String );
    procedure SetApplicationVersion( V :String );
  protected
    constructor Create( anOwner :TComponent ); override;
    function GetPath :string; override;
  published
    property ApplicationName :String
      read FApplicationName write SetApplicationName;
    property ApplicationVersion :String
      read FApplicationVersion write SetApplicationVersion;
    property CompanyName :string
      read FCompanyName write SetCompanyName;
  end;

  TCustomProfile = class( TRegistryEntry )
  private
    FKey :TCustomKey;
  protected
    function GetExist :boolean; override;
    procedure Notification( AComponent :TComponent;
     Operation :TOperation ); override;
    procedure DoDelete; override;
  public
    constructor Create( anOwner :TComponent ); override;
    property Key :TCustomKey
      read FKey write FKey;
  end;

  TVariantProfile = class( TCustomProfile )
  protected
    function GetValue :Variant; virtual;
    procedure SetValue( V :Variant ); virtual;
  public
    property Value :Variant read GetValue write SetValue;
  published
    property Key;
  end;


implementation

{$R ANOTHERREG.RES}

{$I SYSUTILS.INC}
{$I ANOTHERREG.INC}


const

  AutoCloseInterval = 5000;

function CheckCrossRef( Root, Item :TCustomKey ):boolean;
var K :TCustomKey;
begin
  result := false;
  if ( Item = nil ) or ( root = nil ) then exit;
  K := Item;
  while K <> Nil do
   begin
    if K = Root then
     begin
      result := true;
      exit;
     end
    else K := K.FParentKey;
   end;
end;


{ TCustomLey }

procedure TCustomKey.DoDelete;
var H :HKey;
begin
  close;
  if FParentKey <> Nil
    then H := FParentKey.Handle
    else H := HKEY_CURRENT_USER;
  if RegDeleteKey( H, PChar( Path ) ) <> ERROR_SUCCESS
   then raise ERegistryError.CreateResFmt( SDeleteKeyError, [ FullPath ] );
end;

function TCustomKey.GetExist :Boolean;
var H :THandle;
begin
 result := RegOpenKey( HKEY_CURRENT_USER, PChar( FullPath ), H ) = ERROR_SUCCESS;
 if result then RegCloseKey( H );
end;


procedure TCustomKey.Notification( AComponent :TComponent;
 Operation :TOperation );
begin
 if ( Operation = opRemove ) and ( AComponent = FParentKey )
  then FParentKey := nil;
end;


procedure TCustomKey.AutoClose( sender :TObject );
begin
 close;
end;


procedure TCustomKey.SetParentKey( V :TCustomKey );
begin
  if V = FParentKey then exit;
  if CheckCrossRef( Self, V )
   then raise ERegistryError.CreateRes( SCrossRef );
  close;
  FParentKey := V;
end;

procedure TCustomKey.SetActive( V :Boolean );
begin
  if v then Open else Close;
end;

function TCustomKey.GetActive :Boolean;
begin
  result := FHandle <> 0;
end;

procedure TCustomKey.Open;
var H :THandle;
begin
 if ( FHandle = 0 ) then
  begin
   if ( FParentKey <> nil )
     then H := FParentKey.Handle
     else H := HKEY_CURRENT_USER;
   if RegCreateKey( H, PChar( Path ), FHandle ) <> ERROR_SUCCESS
     then raise ERegistryError.CreateResFmt( SOpenKeyError, [ Path ] );
  end;
 if ( FParentKey = nil ) then SetAutoCloseTimer;
end;

procedure TCustomKey.Close;
begin
 if FHandle = 0 then exit;
 if RegCloseKey( FHandle ) <> ERROR_SUCCESS
  then raise ERegistryError.CreateResFmt( SCloseKeyError, [ Path ] );
 FHandle := 0;
 if ( FParentKey = Nil ) then KillAutoCloseTimer;
end;

constructor TCustomKey.Create( anOwner :TComponent );
begin
 inherited Create( anOwner );
 FAutoCloseTimer := Nil;
 FHandle := 0;
 FParentKey := Nil;
end;

destructor TCustomKey.Destroy;
begin
 Close;
 inherited Destroy;
end;

function TCustomKey.GetHandle;
begin
 open;
 result := FHandle;
end;


procedure TCustomKey.SetAutoCloseTimer;
begin
 if FAutoCloseTimer = nil then FAutoCloseTimer := TTimer.Create( Self );
 with FAutoCloseTimer do
  begin
   OnTimer := AutoClose;
   Interval := AutoCloseInterval;
  end;
end;

procedure TCustomKey.KillAutoCloseTimer;
begin
 if FAutoCloseTimer <> nil then
  begin
   FAutoCloseTimer.Enabled := false;
   FAutoCloseTimer.Free;
   FAutoCloseTimer := nil;
  end;
end;

function TCustomKey.GetPath :string;
begin
 if ( FParentKey = Nil )
  then result := LoadStr( SLostKeyPath ) + '\' + Name
  else result := Name;
end;

function TCustomKey.GetFullPath :string;
begin
 if ( FParentKey <> Nil )
  then Result := FParentKey.FullPath + '\' + Path
  else result := path;
end;


{ TAppConfigKey }



procedure TAppConfigKey.SetApplicationName( V :String );
begin
  if V = '' then raise ERegistryError.CreateRes( SInvalidValue );
  close;
  FApplicationName := V;
end;

procedure TAppConfigKey.SetApplicationVersion( V :String );
begin
  if V = '' then raise ERegistryError.CreateRes( SInvalidValue );
  close;
  FApplicationVersion := V;
end;

constructor TAppConfigKey.Create( anOwner :TComponent );
begin
 inherited Create( anOwner );
 FCompanyName := LoadStr( SDefCompanyName );
 FApplicationName := LoadStr( SDefApplicationName );
 FApplicationVersion := LoadStr( SDefApplicationVersion );
end;

function TAppConfigKey.GetPath :string;
begin
 result := 'Software\' + CompanyName + '\' +
  ApplicationName + '\' + ApplicationVersion;
end;

procedure TAppConfigKey.SetCompanyName( V :String );
begin
  if V = '' then raise ERegistryError.CreateRes( SInvalidValue );
  close;
  FCompanyName := V;
end;

{ TCustomProfile }

procedure TCustomProfile.Notification( AComponent :TComponent;
 Operation :TOperation );
begin
  if ( AComponent = FKey ) and ( Operation = opRemove ) then FKey := Nil;
  inherited Notification( AComponent, Operation );
end;

constructor TCustomProfile.Create( anOwner :TComponent );
begin
 inherited Create( anOwner );
 FKey := Nil;
end;

function TCustomProfile.GetExist :boolean;
var T, S :Longint;
begin
  if Key = nil then raise ERegistryError.CreateRes( SNoKey );
  T := REG_BINARY;
  result := RegQueryValueEx( Key.Handle, PChar( Name ),
   nil, @T, nil, @S ) = ERROR_SUCCESS;
end;

procedure TCustomProfile.DoDelete;
begin
  if Key = nil then raise ERegistryError.CreateRes( SNoKey );
  if RegDeleteValue( Key.Handle, PChar( Name ) ) <> ERROR_SUCCESS then
   raise ERegistryError.CreateResFmt( SDeleteValueError, [ Name, Key.Path ] );
end;


{ TVariantProfile }

{ The code looks bizzare, but I had no time and information
 to do something more professional.
 By the way, can anybody make me a present
 called "recent version of Turbo Debugger?" }

function TVariantProfile.GetValue :Variant;
var S, R, T :Longint; B :Pointer;
begin
  if Key = nil then raise ERegistryError.CreateRes( SNoKey );
  T := REG_BINARY;
  R := RegQueryValueEx( Key.Handle, PChar( Name ), nil, @T, nil, @S );
  if ( R <> ERROR_SUCCESS ) then
    begin
     VarClear( result );
     exit;
    end;
  GetMem( B, S + 32 );
  if ( B = Nil ) then raise EOutOfMemory.CreateRes( SOutOfMemory );
  try
    R := RegQueryValueEx( Key.Handle, PChar( Name ), nil, @T, B, @S );
    if ( R <> ERROR_SUCCESS ) then raise ERegistryError.CreateRes( SRegReadError );
    if ( PVarData( B )^.VType = varString )
     then result := String( PChar( Longint( B ) + 2 ) )
     else move( b^, result, sizeof( variant ) );
  finally
    FreeMem( B );
  end;
end;


procedure TVariantProfile.SetValue( V :Variant );
var B :Pointer; R, S :Longint;
begin
  if Key = nil then raise ERegistryError.CreateRes( SNoKey );
  case VarType( V ) of
    varString : S := 2 + Length( V ) + 1;
    varByte, varSmallint, varInteger,
    varSingle, varDouble, varCurrency,
    varBoolean, varDate : S := SizeOf( Variant );
    else raise ERegistryError.CreateRes( SInvalidType );
  end;
  GetMem( B, S + 32 );
  if ( B = Nil ) then raise EOutOfMemory.CreateRes( SOutOfMemory );
  try
    move( V, B^, SizeOf( Variant ) );
    if ( ( VarType( V ) = varString ) and ( Length( V ) > 0 ) ) then
     move( TVarData( V ).VString^,  Pointer( Longint( B ) + 2 )^, S - 2  );
    R := RegSetValueEx( Key.Handle, PChar( Name ), 0, REG_BINARY, B, S );
    if ( R <> ERROR_SUCCESS ) then raise ERegistryError.CreateRes( SWriteError );
  finally
    FreeMem( B );
  end;
end;


{ TRegistryEntry }

procedure TRegistryEntry.Delete;
begin
 if exist then DoDelete;
end;


end.
